#define __PROG_TYPES_COMPAT__
#include <avr/pgmspace.h>
#include <avr/io.h>
#include "uTFT_ST7735.h"
#include <avr/interrupt.h>

#define F_CPU 16000000
#define MOVE_AMOUNT 5
#define LINE_WIDTH 16
#define LINE_HEIGHT 4
#define BRICK_SIZE 7

void myDelay(int16_t ms);

typedef struct {
	int x;
	int y;
	uint16_t color;
} brick;

typedef struct {
	int x;
	int y;
	uint16_t color;
} line;

line control;

typedef enum { 
	STATE_START_MENU,
	STATE_IN_GAME
} state;

uint16_t background_color = Color565(0, 51, 102);
state game_state = STATE_START_MENU;

void draw_brick(brick b) {
	int i, j;
	
	for (i = b.y; i < b.y + BRICK_SIZE; i++) {
		for (j = b.x; j < b.x + BRICK_SIZE; j++) {
			drawPixel(j, i, b.color);
		}
	}
}

void draw_line() {
	int i, j;
	
	for (i = control.y; i < control.y + LINE_WIDTH; i++) {
		for (j = control.x; j < control.x + LINE_HEIGHT; j++) {
			drawPixel(j, i, control.color);
		}
	}
}

void init_brick(brick *b, int x, int y, uint16_t color) {
	b->x = x;
	b->y = y;
	b->color = color;
}

void init_line(line *l, int x, int y, uint16_t color) {
	l->x = x;
	l->y = y;
	l->color = color;
}

void line_move_left() {
	int i, j;
	
	for (i = control.y; i < control.y + MOVE_AMOUNT; i++) {
		for (j = control.x; j < control.x + LINE_HEIGHT; j++) {
			drawPixel(j, i, background_color);
		}
	}
	
	control.y += MOVE_AMOUNT;
	
	for (i = control.y + LINE_WIDTH - MOVE_AMOUNT; i < control.y + LINE_WIDTH; i++) {
		for (j = control.x; j < control.x + LINE_HEIGHT; j++) {
			drawPixel(j, i, control.color);
		}
	}
}

void line_move_right() {
	int i, j;
	
	for (i = control.y + LINE_WIDTH - MOVE_AMOUNT; i < control.y + LINE_WIDTH; i++) {
		for (j = control.x; j < control.x + LINE_HEIGHT; j++) {
			drawPixel(j, i, background_color);
		}
	}
	
	control.y -= MOVE_AMOUNT;
	
	for (i = control.y; i < control.y + MOVE_AMOUNT; i++) {
		for (j = control.x; j < control.x + LINE_HEIGHT; j++) {
			drawPixel(j, i, control.color);
		}
	}
}

void animate_background(uint16_t color) {
	int i, j;
	
	for (i = 0; i < _width / 2 + 1; i++) {
		for (j = 0; j < _height; j++) {
			drawPixel(i, j, color);
			drawPixel(_width - i, j, color);
		}
	}
}

/* intreruperea pentru pin change PA6 si PA7 */
ISR(PCINT0_vect) {
    // PA6 apasat
    if((PINA & (1 << PA6)) == 0)
    {
		if (game_state == STATE_START_MENU) {
			game_state = STATE_IN_GAME;
			animate_background(Color565(0, 51, 102));
		}
		
		else if (game_state == STATE_IN_GAME) {
			if (control.y <= _height - LINE_WIDTH - MOVE_AMOUNT) {
				line_move_left();
			}
		}
    }
    
    // PA7 apasat
    if((PINA & (1 << PA7)) == 0)
    {
		if (game_state == STATE_START_MENU) {
			//Nothing happens
		}
		
		else if (game_state == STATE_IN_GAME) {
			if (control.y >= MOVE_AMOUNT) {
				line_move_right();
			}
		}
    }
}

void IO_init() {
    // butoanele sunt puse ca intrare, cu rezistenta de pull-up
    DDRA &= ~((1 << PA6) | (1 << PA7));
    PORTA |= ((1 << PA6) | (1 << PA7));
    
    // intreruperile sunt activate pentru PCINT8 (PB0) si PCINT9 (PB1)
    PCICR  = (1 << PCIE0);
    PCMSK0 = (1 << PCINT6) | (1 << PCINT7);
}

int get_bit_from_byte(int byte, int bit) {
	return byte & (1 << bit);
}

void display_character(uint16_t x, uint16_t y, char character, uint16_t c_color, uint16_t bg_color) {
  int i, j, byte, starting_index = character * 5;
  
  for (i = 0; i < 8; i++) {
    for (j = 0; j < 5; j++) {
      byte = font[starting_index + j];
	  
      if (get_bit_from_byte(byte, i)) {
			drawPixel(i, j, c_color);
      } 
	  
	  else {
		drawPixel(i, j, bg_color);
      }
    }
  }
}

void display_string(uint16_t x, uint16_t y, char *s, uint16_t s_color, uint16_t bg_color){
	int offset = 0;
	
	while (*s != '\0'){
		display_character(x - offset, y, *s, s_color, bg_color);
		offset += 6;
		s++;
	}
	
}

int main(void){
	// init the 1.8 lcd display
	init();
	
	IO_init();
	
	game_state = STATE_START_MENU;
	
	fillScreen(Color565(0,0,0));
	setTextColor(Color565(255,255,255),Color565(0,0,0));
	setCursor(50,54);
	print("Press left");
	setCursor(56,62);
	print("to start");
	myDelay(150);
	
	sei();
	
	while (game_state == STATE_START_MENU) {
		myDelay(150);
	}
	
	myDelay(500);
	
	char buffer[20];
	int first_time = 1;
	int level = 1;
	int lives = 3;
	int i, j;
	brick bricks[6][19];
	
	while (game_state == STATE_IN_GAME) {
		if (first_time == 1) {
			fillRect(0,0,12,160,Color565(180,50,50));
			
			setCursor(100, 2);
			setTextColor(Color565(235,235,235),Color565(180,50,50));
			sprintf(buffer, "Level %d", level);
			print(buffer);
			
			setCursor(20, 2);
			setTextColor(Color565(235,235,235),Color565(180,50,50));
			sprintf(buffer, "Lives %d", lives);
			print(buffer);
			
			setTextColor(Color565(190,200,210),Color565(0,51,102));
			
			for (i = 3; i >= 1; i--) {
				setCursor(80, 55);
				sprintf(buffer, "%d", i);
				print(buffer);
				myDelay(700);
			}
			
			setCursor(74, 55);
			sprintf(buffer, "Go!");
			print(buffer);
			myDelay(700);
			
			fillRect(55,74,8,18,Color565(0,51,102));
			
			int offset = 0;
			
			for (i = 0; i < 6; i++) {
				for (j = 0 + offset; j < 19 - offset; j++) {
					init_brick(&bricks[i][j], 21 + i * (BRICK_SIZE + 1) , 5 + j * (BRICK_SIZE + 1), Color565(255,255,255));
					
					if (i == 2 || i == 9) {
						bricks[i][j].color = Color565(225,100,0);
					}
					
					draw_brick(bricks[i][j]);
				}
				
				offset += 2;
			}
	
			init_line(&control, 115, 72, Color565(255,0,0));
			draw_line();
			
			first_time = 0;
		}
	}
	
		// COLORS AND 'T'
		
	
		//display_character(50, 50, 'C', Color565(255,0,0), Color565(0,255,0));
		//display_string(50, 50, "Salut", Color565(255,0,0), Color565(0,0,0));
		/*
		fillRect(0,0,127,50,Color565(255,0,0));
		myDelay(5000);
		setTextColor(Color565(255,255,255),Color565(255,00,00));
		setCursor(55,20);
		print("red");
		fillRect(0,50,127,100,Color565(0,255,0));
		setTextColor(Color565(255,255,255),Color565(0,255,00));
		setCursor(50,70);
		print("green");
		fillRect(0,100,127,159,Color565(0,0,255));
		setTextColor(Color565(255,255,255),Color565(0,00,255));
		setCursor(55,120);
		print("blue");
		myDelay(5000);
		drawRect(5,5,118,150,Color565(255,255,255));
		
		myDelay(5000);
 
		fillScreen(Color565(0,0,0));
		uint8_t y;
		for (y=0; y<160; y+=8)
		{
			drawLine(0, 0, 127, y, Color565(255,0,0));	
			drawLine(0, 159, 127, y, Color565(0,255,0));
			drawLine(127, 0, 0, y, Color565(0,0,255));
			drawLine(127, 159, 0, y, Color565(255,255,255));
		}
		myDelay(5000);
		
		// TEXT		
		fillScreen(Color565(0,0,0));
		setCursor(0,0);
		setTextWrap(1);
*/
		
		/*setTextColor(Color565(200,200,255),Color565(50,50,50));
		unsigned char i;
		char ff[]="a";
		for (i=32; i<128; i++) 
		{
			ff[0]=i;
			print(ff);
		}
		myDelay(5000);*/

	return 0;
}